@@ -1,6 +1,6 @@
 /*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
  *
- *  Use of this source code is governed by a GPLv3+/MPLv2+ license that can
+ *  Use of this source code is governed by a BSD-style license that can
  *  be found in the License.html file in the root of the source tree.
  */
 
@@ -79,6 +79,7 @@ Core::Core() : policies(this), reports(this)
     watch_folders_manager = new WatchFoldersManager(this);
     policies.create_values_from_csv();
     compression_mode = MediaConchLib::compression_ZLib;
+    no_database = false;
 }
 
 Core::~Core()
@@ -154,18 +155,21 @@ void Core::load_plugins_configuration()
 void Core::load_database()
 {
 #ifdef HAVE_SQLITE
-    std::string db_path;
-    if (!config || config->get("SQLite_Path", db_path) < 0)
-        db_path = get_database_path();
-
-    db = new SQLLiteReport;
-
-    ((Database*)db)->set_database_directory(db_path);
-    db->set_database_filename(database_name);
-    if (db->init_report() < 0)
+    if (!no_database)
     {
-        delete db;
-        db = NULL;
+        std::string db_path;
+        if (!config || config->get("SQLite_Path", db_path) < 0)
+            db_path = get_database_path();
+
+        db = new SQLLiteReport;
+
+        ((Database*)db)->set_database_directory(db_path);
+        db->set_database_filename(database_name);
+        if (db->init_report() < 0)
+        {
+            delete db;
+            db = NULL;
+        }
     }
 #endif
     if (!db)
@@ -224,7 +228,7 @@ const std::string& Core::get_implementation_schema_file()
 }
 
 //---------------------------------------------------------------------------
-void Core::create_default_implementation_schema()
+int Core::create_default_implementation_schema(std::string& err)
 {
     std::string path = get_local_data_path();
     std::string file = path + "MatroskaSchema.xml";
@@ -233,12 +237,17 @@ void Core::create_default_implementation_schema()
     ofs.open(file.c_str(), std::ofstream::out | std::ofstream::binary);
 
     if (!ofs.is_open())
-        return;
+    {
+        err = "Unable to write implementation schema file.";
+        return -1;
+    }
 
     ofs << xsl_schema_matroska_schema;
     ofs.close();
 
     set_implementation_schema_file(file);
+
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -259,6 +268,18 @@ void Core::set_compression_mode(MediaConchLib::compression compress)
     compression_mode = compress;
 }
 
+//---------------------------------------------------------------------------
+void Core::set_no_database(bool ndb)
+{
+    no_database = ndb;
+    if (db)
+    {
+        delete db;
+        db = NULL;
+    }
+    load_database();
+}
+
 //---------------------------------------------------------------------------
 int Core::get_ui_poll_request() const
 {
@@ -396,6 +417,13 @@ long Core::checker_analyze(int user, const std::string& file, bool& registered,
 
     std::string options_str = serialize_string_from_options_vec(options);
     id = file_is_registered_and_analyzed(user, file, analyzed, options_str, err);
+
+    if (id >= 0 && analyzed && mil_analyze) // check report's library version
+    {
+        if (get_db()->version_registered(user, id, err) != mil_version())
+            force_analyze = true;
+    }
+
     if (force_analyze)
         analyzed = false;
 
@@ -477,6 +505,14 @@ long Core::checker_analyze(int user, const std::string& filename, long src_id, s
     return id;
 }
 
+//--------------------------------------------------------------------------
+int Core::checker_list_mediainfo_outputs(std::string& output, std::string&)
+{
+    output = Ztring(MI->Option(__T("Info_OutputFormats_JSON"))).To_UTF8();
+
+    return 0;
+}
+
 //---------------------------------------------------------------------------
 int Core::file_add_generated_file(int user, long src_id, long generated_id, std::string& err)
 {
@@ -576,7 +612,10 @@ int Core::checker_file_information(int user, long id, MediaConchLib::Checker_Fil
     db_mutex.Leave();
 
     if (ret == 0)
+    {
+        info.existing = file_is_existing(info.filename);
         info.options = parse_options_vec_from_string(options);
+    }
     return ret;
 }
 
@@ -785,7 +824,7 @@ void Core::register_report_xml_to_database(int user, long file, const std::strin
     std::string err;
     db_mutex.Enter();
     get_db()->save_report(user, file, report_kind, MediaConchLib::format_Xml,
-                          options, new_report, mode, true, err);
+                          options, new_report, mode, mil_version(), err);
     db_mutex.Leave();
 }
 
@@ -802,7 +841,7 @@ void Core::register_report_mediainfo_text_to_database(int user, long file, Media
     std::string err;
     db_mutex.Enter();
     get_db()->save_report(user, file, MediaConchLib::report_MediaInfo, MediaConchLib::format_Text,
-                          "", report, mode, true, err);
+                          "", report, mode, mil_version(), err);
     db_mutex.Leave();
 }
 
@@ -818,7 +857,7 @@ void Core::register_report_mediainfo_xml_to_database(int user, long file, MediaI
     std::string err;
     db_mutex.Enter();
     get_db()->save_report(user, file, MediaConchLib::report_MediaInfo, MediaConchLib::format_Xml,
-                          "", report, mode, true, err);
+                          "", report, mode, mil_version(), err);
     db_mutex.Leave();
 }
 
@@ -853,7 +892,7 @@ void Core::register_report_micromediatrace_xml_to_database(int user, long file,
     std::string err;
     db_mutex.Enter();
     get_db()->save_report(user, file, MediaConchLib::report_MicroMediaTrace, MediaConchLib::format_Xml,
-                          "", report, mode, true, err);
+                          "", report, mode, mil_version(), err);
     db_mutex.Leave();
 }
 
@@ -914,6 +953,43 @@ int Core::register_mediaconch_to_database(int user, long file, const std::string
     return ret;
 }
 
+//---------------------------------------------------------------------------
+
+int Core::transform_mixml_report(const std::string& mi_xml, const std::string& mi_inform,
+                                 const std::vector<std::pair<std::string,std::string> >& options,
+                                 std::string& report, std::string& err)
+{
+    MediaInfoNameSpace::MediaInfo* tmpMI=new MediaInfoNameSpace::MediaInfo;
+
+    tmpMI->Option(__T("Details"), __T("0"));
+    tmpMI->Option(__T("Inform"), Ztring().From_UTF8(mi_inform));
+
+
+    for (size_t i = 0; i < options.size(); ++i)
+    {
+        if (!options[i].first.empty() && !options[i].second.empty())
+            tmpMI->Option(Ztring().From_UTF8(options[i].first),  Ztring().From_UTF8(options[i].second));
+    }
+
+    size_t BufferSize = strlen(mi_xml.data());
+
+    if (BufferSize == 0)
+    {
+        err = "Original report is empty.";
+        return -1;
+    }
+
+    tmpMI->Open_Buffer_Init(BufferSize);
+    tmpMI->Open_Buffer_Continue((const ZenLib::int8u*)mi_xml.data(), BufferSize);
+    tmpMI->Open_Buffer_Finalize();
+
+    report = Ztring(tmpMI->Inform()).To_UTF8();
+
+    delete tmpMI;
+
+    return 0;
+}
+
 //---------------------------------------------------------------------------
 int Core::get_report_saved(int user, const std::vector<long>& files,
                            MediaConchLib::report reportKind, MediaConchLib::format f,
@@ -927,9 +1003,8 @@ int Core::get_report_saved(int user, const std::vector<long>& files,
         return 0;
     }
 
-    if (reportKind != MediaConchLib::report_MediaConch &&
-        reportKind != MediaConchLib::report_MediaInfo && reportKind != MediaConchLib::report_MicroMediaTrace
-        && reportKind != MediaConchLib::report_MediaVeraPdf && reportKind != MediaConchLib::report_MediaDpfManager)
+    if (reportKind != MediaConchLib::report_MediaConch && reportKind != MediaConchLib::report_MediaInfo && reportKind != MediaConchLib::report_MicroMediaTrace &&
+        reportKind != MediaConchLib::report_MediaVeraPdf && reportKind != MediaConchLib::report_MediaDpfManager && reportKind != MediaConchLib::report_MediaImsc1Validation)
     {
         err = "Report kind asked is not recognized.";
         return -1;
@@ -1287,6 +1362,21 @@ std::string Core::get_local_data_path()
     if (!ZenLib::Dir::Exists(z_path))
         ZenLib::Dir::Create(z_path);
 
+#if defined(UNIX) || defined(MACOS) || defined(MACOSX)
+    if (!ZenLib::Dir::Exists(z_path))
+    {
+        const char* tmpdir;
+        if ((tmpdir = getenv("TMPDIR")) == NULL)
+            tmpdir = "/tmp";
+
+        std::string tmp_local_path = std::string(tmpdir) + separator + "MediaConch";
+        z_path = ZenLib::Ztring().From_UTF8(tmp_local_path);
+        if (ZenLib::Dir::Exists(z_path) || ZenLib::Dir::Create(z_path))
+            local_path = tmp_local_path;
+    }
+#endif
+
+
     return local_path;
 }
 
@@ -1557,6 +1647,62 @@ std::string Core::get_date()
     return str;
 }
 
+//--------------------------------------------------------------------------
+unsigned long long Core::mil_version()
+{
+    std::string version_string = ZenLib::Ztring(Menu_Option_Preferences_Option (__T("Info_Version"), __T(""))).To_UTF8();
+
+    std::string search(" - v");
+    size_t pos = version_string.find(search);
+    if (pos != std::string::npos)
+        version_string = version_string.substr(pos + search.length());
+
+    search = std::string(".");
+    const char* version = version_string.c_str();
+
+
+    long major = 0;
+    long minor = 0;
+    long patch = 0;
+    long build = 0;
+    char *end = NULL;
+
+    major = strtol(version, &end, 10);
+    if (end && *end != '\0')
+    {
+        version = ++end;
+        minor = strtol(version, &end, 10);
+        if (minor > 4095 && (!end || *end == '\0'))
+        {
+            build = minor;
+            minor = 0;
+            patch = 0;
+        }
+    }
+
+    if (end && *end != '\0')
+    {
+        version = ++end;
+        patch = strtol(version, &end, 10);
+        if (patch > 255 && (!end || *end == '\0'))
+        {
+            build = patch;
+            patch = 0;
+        }
+    }
+
+    if (end && *end != '\0')
+    {
+        version = ++end;
+        build = strtol(version, &end, 10);
+    }
+
+    return (((unsigned long long )major << 52) & 0xFFF0000000000000) |
+           (((unsigned long long )minor << 40) & 0x000FFF0000000000) |
+           (((unsigned long long )patch << 32) & 0x000000FF00000000) |
+           (((unsigned long long )build)       & 0x00000000FFFFFFFF);
+}
+
 //--------------------------------------------------------------------------
 void Core::plugin_add_log(int level, const std::string& log)
 {
